/*-----------------------------------------------------------------------------*
// include
*-----------------------------------------------------------------------------*/
#include "harktool.h"




/*-----------------------------------------------------------------------------*
// extern
*-----------------------------------------------------------------------------*/
extern struct GLOBAL  g;








/*-----------------------------------------------------------------------------*
* TSP録音から伝達関数作成処理

*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfrec()
{


    harktool_LogI("harktoolcli_calctfrec start");



    if (strlen(g.argv.mic_file) == 0) {
        harktool_LogE("argument error　[--mic-list]");
        harktool_help("calctfrec");
        harktool_Exit(EXIT_FAILURE);
    } else if (strlen(g.argv.tsp_file) == 0) {
        harktool_LogE("argument error　[--tsp-list]");
        harktool_help("calctfrec");
        harktool_Exit(EXIT_FAILURE);

    } else if (strlen(g.argv.output) == 0) {
        harktool_LogE("argument error　[--output-file]");
        harktool_help("calctfrec");
        harktool_Exit(EXIT_FAILURE);

    } else if (g.argv.tf_type != LOC && g.argv.tf_type != SEP && g.argv.tf_type != LOC_SEP) {
        harktool_LogE("argument error　[--output-type]");
        harktool_help("calctfrec");
        harktool_Exit(EXIT_FAILURE);
    }



    /*--------------------------------------------------*/
    //伝達関数作成
    /*--------------------------------------------------*/
    harktoolcli_calctfrecExec();


    /*--------------------------------------------------*/
    // end proc
    /*--------------------------------------------------*/
    harkio_Log_printf(harkio_Log_Info, "@ harktoolcli_calctfrec normal end");

    harktool_Exit(EXIT_SUCCESS);


}//end





/*-----------------------------------------------------------------------------*
* TSP録音から伝達関数作成処理
*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfrecExec()
{



    /*--------------------------------------------------*/
    // micファイル(microphones.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.mic_xml = harkio_XML_newFromXML(g.argv.mic_file , "");



    /*--------------------------------------------------*/
    //srcファイル(source.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.tsp_xml = harkio_XML_newFromXML(g.argv.tsp_file , "");


    /*--------------------------------------------------*/
    // position type
    /*--------------------------------------------------*/
    g.PosType = TSP;


    /*--------------------------------------------------*/
    //mic数
    /*--------------------------------------------------*/
    g.Nmic = harkio_Positions_getSize(g.mic_xml->poses);


    /*--------------------------------------------------*/
    //音源数
    /*--------------------------------------------------*/
    g.Nsrc = harkio_Positions_getSize(g.tsp_xml->poses);


    /*--------------------------------------------------*/
    // Config取得
    /*--------------------------------------------------*/
    g.Nfft           = harkio_Config_getNfft(g.tsp_xml->cfg);
    g.TspLength      = harkio_Config_getTSPLength(g.tsp_xml->cfg);
    g.SamplingRate   = harkio_Config_getSamplingRate(g.tsp_xml->cfg);
    g.SignalMax      = harkio_Config_getSignalMax(g.tsp_xml->cfg);
    g.SyncAverage    = harkio_Config_getSynchronousAverage(g.tsp_xml->cfg);
    g.OrgTspPath     = harkio_Config_getTSPpath(g.tsp_xml->cfg);
    g.TspOffset      = harkio_Config_getTSPoffset(g.tsp_xml->cfg);
    g.PeakSearchFrom = harkio_Config_getPeakSearchFrom(g.tsp_xml->cfg);
    g.PeakSearchTo   = harkio_Config_getPeakSearchTo(g.tsp_xml->cfg);


    g.IRsize = g.Nsrc * g.Nmic * g.Nfft ;
    g.TFsize = g.Nsrc * g.Nmic * (g.Nfft/2 + 1) ;


    // error check
    if (g.OrgTspPath == NULL) {
        fprintf(stderr, "TSPpath is empty in %s\n", g.argv.tsp_file);
        exit(1);
    }


    /*--------------------------------------------------*/
    /* 領域確保                                         */
    /*--------------------------------------------------*/
    // impulse応答用領域
    // npos(72) x Nmic(8) x Nfft(512) x sizeof(float) 分の領域確保
    harktool_new(IR_VECT);


    // impulse応答 計算用領域
    // npos(72) x Nmic(8) x Nfft(512) x sizeof(float) 分の作業領域確保
    harktool_new(TMP_IR_VECT);


    // impulse応答 BACKUP領域
    harktool_new(BACK_IR_VECT);


    // 伝達関数用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の領域確保
    harktool_new(TF_VECT);


    // 伝達関数 作業用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
    harktool_new(TMP_TF_VECT);


    //伝達関数作成用TEMP領域
    harktool_new(BUFFER);


    //Mic position
    harktool_new(MIC_POS_VECT);

    //SRC position
    harktool_new(SRC_POS_VECT);

    //SORT
    harktool_new(SORT_POS_ID);

    // インパルス応答ファイルのPATH格納領域
    harktool_new(FLT_PATH);

    // TSP wavファイル名格納領域
    harktool_new(TSP_WAV_NAME);



    /*--------------------------------------------------*/
    //  FFT data set
    /*--------------------------------------------------*/
    setFftData();



    /*--------------------------------------------------*/
    /*  Create Directory                                */
    /*--------------------------------------------------*/
    harktool_createTmpPath() ;




    /*--------------------------------------------------*/
    // id を昇順に並べ替えてpositionをセット
    /*--------------------------------------------------*/
    //MIC ID 並べ替え
    harktool_setMicPosId();
    //mic positionセット + 極座標->直交座標変換処理
    harktool_setMicPos();


    // TSP ID 並べ替え
    harktool_setTspPosId();



    //tsp positionセット + 極座標->直交座標変換処理
    harktool_setSrcPos();


    /*--------------------------------------------------*/
    // ChannelList init
    /*--------------------------------------------------*/
    harktool_setChannelList();


    /*--------------------------------------------------*/
    /* calcSysimpでIR変換                               */
    /* poses->pos.pathをcalcSysimpにわたす                      */
    /* tsp_xml->poses->pos[i].path //positionごとのファイルのパス */
    /*--------------------------------------------------*/
    char buf[10];
    int tsp_id;
    harkio_Position* pos;
    int src;
    int i;



    for (src=0 ; src<g.Nsrc ; src++) {
        tsp_id = g.TspPosId[src] ;

        //tsp_idのpositionを取得
        pos = harkio_Positions_getById(g.tsp_xml->poses , tsp_id);


        /*--------------------------------------------------*/
        //position毎にtempディレクトリ作成
        /*--------------------------------------------------*/
        char *str;
        char strPath[PATH_LEN];

        str = strrchr(pos->path ,'/');

        if (str!=NULL) {
            //wav名を格納
            sprintf(g.tmp.tsp_wav_name[src] , "%s" , str+1);

            // /tmp/harktool/日時/tsp/xxx.wav ディレクトリ作成
            memset(strPath  , '\0'    , sizeof(strPath));
            sprintf(strPath , "%s/%s" , g.tmp.tsp_path , str+1);

            if (harktool_CheckDir(strPath) == NON_DIR_TYPE) {
                harktool_createDir(strPath);
            }
        } else {
            char msg[MSG_LEN];
            sprintf(msg, "can not get path [%s] ", pos->path);
            harktool_LogE(msg);
            harktool_Exit(EXIT_FAILURE);
        }

        //path copy
        sprintf(g.tmp.flt_path[src] , "%s" ,  strPath);  //fltファイル格納先path


        /// outimpfilehead = "/tmp/harktool/xxx.wav/float"
        sprintf(g.tmp.outimpfilehead , "%s/%s"  ,  g.tmp.flt_path[src] , FILE_PREFIX);

        /*--------------------------------------------------*/
        /* channellist 設定                                 */
        /*--------------------------------------------------*/

        if (pos->numChannels != 0) {
            memset(g.ChannelList , '\0' , sizeof(g.ChannelList));

            for (i=0; i< pos->numChannels; i++) {
                sprintf(buf , "%d,", pos->channels[i]);
                strcat(g.ChannelList , buf);
            }

            g.ChannelList[ strlen(g.ChannelList)-1 ] = 0;  //最後のカンマを削除

        }

        /*--------------------------------------------------*/
        /* calcsSysImp実行                                  */
        /*--------------------------------------------------*/
        Print_calcSysImp(pos->path);

        harktool_calcSysImp(15
                            // , "--tsporigflt"     , "/usr/bin/harktool4_utils/16384.little_endian.tsp"
                            , "--tsporigwav"     , g.OrgTspPath           // "/usr/bin/harktool4_utils/16384.little_endian.wav"
                            , "--sigmax"         , g.SignalMax         // "32768"
                            , "--syncaddfreq"    , g.SyncAverage       // "16"
                            , "--syncaddoffset"  , g.TspOffset         // "16384"
                            , "--outchannellist" , g.ChannelList             //"0,1,2,3,4,5,6,7"
                            , "--inputwave"      , pos->path               // "/home/circle360_5deg_H115_R150_D180_1.wav"
                            , "--outputpath"     , g.tmp.outimpfilehead);  // "/tmp/harktool/20140101/tsp/circle360_5deg_H115_R150_D180_1.wav/float"




        //Nmic分　IRvectorにIRデータをセットする
        harktool_setIRvectFromFlt(src);

        fprintf(stderr, "progress:%d / %d\n", src , g.Nsrc);

    }//src_for_end


    //IRvector back up
    harktool_copyIRvectToBackIRvect();


    switch (g.argv.tf_type) { //tf_type:SEP,LOC,LOC_SEP
    case LOC:
        // 残響音と思われる部分に0を書き込む
        harktool_EraseReverberation();


        // 残響音が直接音より大きい可能性がある音源を表示
        harktool_ShowReverbSources();

        // インパルス応答をFFTして伝達関数を取得
        harktool_FFT();


        //定位伝達関数の配列からharkio_Matrix型に変換する
        harktool_setLoctfFromArray();


        break;


    case SEP:
        harktool_genGSS();
        break;

    case LOC_SEP:
        //------------------------------------//
        //Loc
        //------------------------------------//
        // 残響音と思われる部分に0を書き込む
        harktool_EraseReverberation();

        // 残響音が直接音より大きい可能性がある音源を表示
        harktool_ShowReverbSources();

        // インパルス応答をFFTして伝達関数を取得
        harktool_FFT();

        //定位伝達関数の配列からharkio_Matrix型に変換する
        harktool_setLoctfFromArray();



        //data free
        harktool_delete(IR_VECT);
        harktool_delete(TMP_IR_VECT);
        harktool_delete(TF_VECT);
        harktool_delete(TMP_TF_VECT);
        harktool_delete(BUFFER);

        harktool_new(IR_VECT);       // impulse応答用領域
        harktool_new(TMP_IR_VECT);   // impulse応答 計算用領域
        harktool_new(TF_VECT);       // 伝達関数用領域
        harktool_new(TMP_TF_VECT);   // 伝達関数 作業用領域
        harktool_new(BUFFER);        // 伝達関数作成用TEMP領域


        //recovery
        harktool_copyBackIRvectToIRvect();

        //------------------------------------//
        //sep
        //------------------------------------//
        harktool_genGSS();
        break;

    }


    /*--------------------------------------------------*/
    // 伝達関数をZipファイルに出力
    //
    /*--------------------------------------------------*/

    harktool_writeTF();


    /*--------------------------------------------------*/
    //
    /*--------------------------------------------------*/

    createReadMeToZipFile(g.argv.output);




#if 0
    // インパルス応答の正規化
    switch (g.argv.tf_type) {
    case SEP: // GSS

        harktool_normalizeIR();
        break;


    case LOC:// MUSIC

        // 残響音と思われる部分に0を書き込む
        harktool_EraseReverberation();

        // 残響音が直接音より大きい可能性がある音源を表示
        harktool_ShowReverbSources();

        break;

    default:
        harktool_LogE("tf_type error");
        harktool_Exit(EXIT_FAILURE);
    }



    // インパルス応答をFFTして伝達関数を取得
    harktool_FFT();






    /*  conj                                            */
    if (g.argv.tf_type == SEP) { // GSS

        harktool_conjTF();


        harktool_normalizeTF();
    }




    /*--------------------------------------------------*/
    /*  harkio_Matrix型に変換する                       */
    /*--------------------------------------------------*/
    switch (g.argv.tf_type) {
    case SEP: // GSS
        //分離伝達関数の配列からharkio_Matrix型に変換する
        harktool_setSeptfFromArray();
        break;

    case LOC:// MUSIC
        //定位伝達関数の配列からharkio_Matrix型に変換する
        harktool_setLoctfFromArray();
        break;
    }





    /*--------------------------------------------------*/
    // 伝達関数をZipファイルに出力
    //
    /*--------------------------------------------------*/
    harktool_writeTF();


    /*--------------------------------------------------*/
    //
    /*--------------------------------------------------*/
    createReadMeToZipFile(g.argv.output);
#endif

}//main end








